using System;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Azure.Cosmos.Table;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.Http;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace Perficient.Blog
{
    public class PipelineRuns
    {
        #region private settings
        private const string CONNECTION_STRING = "*****************";
        private const string TABLE_NAME = "adfRuns";
        #endregion

        private readonly ILogger<PipelineRuns> _logger;
        private CloudTable _table;

        public PipelineRuns(ILogger<PipelineRuns> logger)
        {
            _logger = logger;
            _table = GetCloudTable();
        }

        [FunctionName("LogPipelineRun")]
        public async Task<IActionResult> LogPipelineRun(
            [HttpTrigger(AuthorizationLevel.Admin, "post", Route = null)] HttpRequestMessage req,
            ILogger log)
        {
            var reqBody = await req.Content.ReadAsStringAsync();
            var reqObject = JsonConvert.DeserializeObject<PipelineRunEntry>(reqBody);

            if (reqObject == null 
                || string.IsNullOrEmpty(reqObject.PartitionKey) 
                || string.IsNullOrEmpty(reqObject.RowKey)
                || string.IsNullOrEmpty(reqObject.Body))
            {
                _logger.LogError("Invalid LogPipelineRun function request, expected body: { 'partitionKey':'<Pipeline Name>', 'rowKey':'<Run Id>', 'body':'<Body>' }");
                return new BadRequestResult();
            }

            var entity = new PipelineRunEntry(reqObject.PartitionKey, reqObject.RowKey, reqObject.Body);
            var record = await InsertTableEnityAsync(entity);
            return new OkObjectResult(record);
        }

        [FunctionName("GetPipelineRun")]
        public async Task<IActionResult> GetPipelineRun(
            [HttpTrigger(AuthorizationLevel.Admin, "post", Route = null)] HttpRequestMessage req,
            ILogger log)
        {
            var reqBody = await req.Content.ReadAsStringAsync();
            var reqObject = JsonConvert.DeserializeObject<PipelineRunEntry>(reqBody);

            if (reqObject == null || string.IsNullOrEmpty(reqObject.PartitionKey) || string.IsNullOrEmpty(reqObject.RowKey))
            {
                _logger.LogError("Invalid GetPipelineRun function request, expected body: { 'partitionKey':'<Pipeline Name>', 'rowKey':'<Run Id>' }");
                return new BadRequestResult();
            }

            var record = await GetTableEntityAsync(reqObject.PartitionKey, reqObject.RowKey);
            return new OkObjectResult(record);
        }

        private async Task<PipelineRunEntry> InsertTableEnityAsync(PipelineRunEntry entry)
        {
            try
            {
                TableOperation insertOrMergeOperation = TableOperation.InsertOrMerge(entry);
                TableResult result = await _table.ExecuteAsync(insertOrMergeOperation);
                PipelineRunEntry record = result.Result as PipelineRunEntry;

                return record;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error upserting storage table entity: {entry}");

                return entry;
            }
        }

        private async Task<PipelineRunEntry> GetTableEntityAsync(string partitionKey, string rowKey)
        {
            try
            {
                TableOperation retrieveOperation = TableOperation.Retrieve<PipelineRunEntry>(partitionKey, rowKey);
                TableResult result = await _table.ExecuteAsync(retrieveOperation);
                PipelineRunEntry record = result.Result as PipelineRunEntry;

                return record;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error getting storage table entity for partitionKey: {partitionKey}, rowKey: {rowKey}");

                return default;
            }
        }

        private CloudTable GetCloudTable()
        {
            return CloudStorageAccount.Parse(CONNECTION_STRING)
                .CreateCloudTableClient()
                .GetTableReference(TABLE_NAME);
        }
    }

    public class PipelineRunEntry : TableEntity
    {
        public PipelineRunEntry()
        {}

        public PipelineRunEntry(string pipelineName, string runId, string body)
        {
            PartitionKey = pipelineName;
            RowKey = runId;
            Body = body;
        }

        public string Body { get; set; }
    }
}
